% Inspect the integration point quantities.
%
% function idat = inspect_integration_points(self, ...
%         geom,  Temp, gcell_list, context,...
%         inspector, idat)
%
% Input arguments
%    geom - reference geometry field
%    Temp - temperature field
%    gcell_list - indexes of the geometric cells that are to be inspected:
%          The gcells to be included are: gcells(gcell_list).
%    context    - structure: see the update() method.
%    inspector - function handle or in-line function with the signature
%             idat =inspector(idat, out, xyz, pc),
%        where
%     idat - a structure or an array that the inspector may
%                use to maintain some state, for instance minimum or
%                maximum of stress, out is the output  of the update()
%                method, and xyz is the location of the integration point
%                in the *reference* configuration. The argument pc are the
%                parametric coordinates of the quadrature point.
% Output arguments
%     idat - see the description of the input argument
%
function idat = inspect_integration_points(self, ...
        geom, Temp, gcell_list, context,...
        inspector, idat)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Material
    mat = get(self.feblock, 'mater');
    matstates = get(self, 'matstates');
    context.strain= [];
    conns = get(gcells, 'conn'); % connectivity
   % Now loop over all gcells in the block
    for m=1:length(gcell_list)
        i=gcell_list(m);
        conn =conns(i,:);
        X = gather(geom, conn, 'values', 'noreshape'); % coord
        T = gather(Temp, conn, 'values'); % displ
        % Loop over all integration points
        for j=1:npts_per_gcell
            N = bfun (gcells, pc(j,:));
            Rm=material_directions(self,gcells,pc(j,:),X);% do I need this?
            Nder = bfundpar (gcells, pc(j,:));
            NderX = bfundsp (gcells, Nder, X);
            context.ms=matstates{i,j};
            context.xyz=N'*X;
            context.gradtheta =  T'*NderX;
            [out,ignore] = update(mat, matstates{i,j}, context);
            if ~isempty (inspector)
                idat =feval(inspector,idat,out,N'*X,pc(j,:));
            end
        end
    end
    return;
end
