% Constructor of the finite element block class to be used with a
% grounding spring elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Parameters: those recognized by feblock
%
function retobj = feblock_defor_ground (varargin)
    class_name='feblock_defor_ground';
    if (nargin == 0)
        parent = feblock;
        self.spring_constant=  1.0;
        self.stiffness=  {};
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent = feblock(varargin{:});
            self.spring_constant =[];
            if (isfield(Parameters,'spring_constant'))
                self.spring_constant = Parameters.spring_constant;
            end
            self.stiffness ={};
            if (isfield(Parameters,'stiffness'))
                self.stiffness = Parameters.stiffness;
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
