% Compute the stiffness matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, u1, u)
%     geom=reference geometry field
%     u1=displacement field in the *current* configuration
%     u=displacement field in the last known configuration
%    Returns an array of elemat objects that may be assembled
%    into the system stiffness matrix.
%
function ems = stiffness (self, geom, u)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
    Kes= cell(ngcells,1);
    eqnums = cell(ngcells,1);
    if (isempty(self.stiffness))
        Ke=diag(self.spring_constant);
    end
    for i=1:ngcells
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        if (isempty(self.stiffness))
            Kes{i} = Ke;
        else
            Kes{i} = self.stiffness{i};
        end
    end
    ems  = elematset(struct('mat',{Kes}, 'eqnums',{eqnums}));
end
