% Constructor of the finite element block class to be used with a
% three-dimensional solid elements.
%    This class implements large displacement, large strain mechanics.
%    for continuum, isoparametric elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
%
function retobj = feblock_defor_nonlinear (varargin)
    class_name='feblock_defor_nonlinear';
    if (nargin == 0)
        parent = feblock;
        self.hBlmat= [];
        self.matstates = [];
        self.Rm= [];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock(varargin{:});
            self.hBlmat= [];
            self.matstates = [];
            gcells = get(parent,'gcells');
            if (count(gcells) < 1)
                error('Need at least one gcell!');
            end
            integration_rule = get(parent,'integration_rule');
            if ~isempty(integration_rule)
                npts_per_gcell = get(integration_rule,'npts_per_gcell');
                self.matstates = cell(count(gcells),npts_per_gcell);
                for i=1:count(gcells)
                    for j=1:npts_per_gcell;
                        self.matstates{i,j} = newmatstate(get(parent,'mater'));
                    end
                end
            else
                self.matstates = [];
            end
            dim= get(gcells,'dim');
            switch dim
                case 1
                    self.hBlmat=@feblock_defor_nonlinear_Blmat1;
                case 2
                    if get(gcells,'axisymm')
                        self.hBlmat=@feblock_defor_nonlinear_Blmat2axisymm;
                    else
                        self.hBlmat=@feblock_defor_nonlinear_Blmat2;
                    end
                case 3
                    self.hBlmat=@feblock_defor_nonlinear_Blmat3;
                otherwise
                    error (' Not implemented');
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
