%  Draw graphic representation of local coordinate system for all beam gcells.
%
% function draw_local_coordinates(self, gv, context)
%
%
% Input arguments
% self = self
% gv = graphic viewer
% context = struct
% with mandatory fields
%    x=reference geometry field
%    u=displacement field
% and with optional fields
%    length = length of the arrows
%
function ems = draw_local_coordinates (self, gv, context)
    if (isfield(context,'length'))
        a=context.length;
    else
        a=1;
    end
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(context.x,'dim');
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Xs =gather(context.x,(1:get(context.x,'nfens')),'values','noreshape');
    U1s =gather(context.u,(1:get(context.u,'nfens')),'values','noreshape');
    if (isfield(context,'rot'))
        context.R = update_rotation_field(field(struct ('name',['R'], 'dim', 9, 'data', ones(get(context.rot,'nfens'),1)*reshape(eye(3),1,9))),context.rot);
    end
    Rs =gather(context.R,(1:get(context.R,'nfens')),'values','noreshape');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % coordinates of nodes
        U1 = U1s(conn,:); % coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);
        RJ =reshape(Rs(conn(2),:),3,3);
        x1 = x0+U1s(conn,:); % coordinates of nodes
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        Center=mean(x1);
        options.color='red';
        draw_arrow(gv, Center, a*T(:,1), options);
        options.color='green';
        draw_arrow(gv, Center, a*T(:,2), options);
        options.color='blue';
        draw_arrow(gv, Center, a*T(:,3), options);
    end
end