% Compute the damper absorbing boundary condition matrices of the individual gcells.
% Return an array of them so they may be assembled.
%    Call as
%       ems = surface_damper_abc(feb, geom, v)
%    where
%       geom=geometry field
%       v=velocity field
%
function ems = surface_damper_abc(self, geom, v)
    gcells = get(self.feblock_defor_ss,'gcells');
    nfens = get(gcells,'nfens');
    dim = get(geom,'dim');
    % Integration rule
    integration_rule = get(self.feblock_defor_ss, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    w  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    % Material
    mat = get(self.feblock_defor_ss, 'mater');
    % damping coefficient 
    h = self.damping_abc;
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ke = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(v,conns(i,:),'eqnums');
        Ke{i} =zeros(get(geom,'dim')*nfens);
    end
    xs =gather(geom,(1:get(geom,'nfens')),'values','noreshape');
    for i=1:size(conns,1)
        conn =conns(i,:);
        x=xs(conn,:);
        He = zeros(nfens*dim); % element matrix
        ns=zeros(nfens*dim, 1);
        for j=1:npts_per_gcell
            c =Ns{j}'*x;% physical location of the quadrature point
            J = x' * Nders{j};% We compute the Jacobian matrix
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,J);
            end
            Jac = Jacobian_surface(gcells,conn, Ns{j}, J, x);
            if (Jac<=0),error;;end
            n=normal(self,c,J);
            for k= 1:nfens 
                ns((k-1)*dim+1:k*dim)=n*Ns{j}(k);
            end 
            He = He + ns*ns' * (h* Jac * w(j));
            Ke{i} = Ke{i} + He;
        end
    end
    ems  = elematset(struct('mat',{Ke}, 'eqnums',{eqnums}));
end
