% Compute the mass matrices of the individual gcells.
%
% function ems = mass (self, geom, w)
%
% Return an array of them so they may be assembled.
%   Call as:
%     ems = mass(feb, geom, w);
%   geom=geometry field
%   w=displacement field
%
function ems = mass (self, geom, w)
    gcells = get(self.feblock,'gcells');
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
    Mes = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    weights  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    % Precompute base function values and their parametric derivatives
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    mu = get(self, 'mu');
    % Now loop over all gcells in the block
    for i=1:size(conns, 1)
        conn = conns(i,:); % connectivity
        eqnums{i} =gather(w,conn,'eqnums');
        x = gather(geom, conn, 'values', 'noreshape'); % coordinates of nodes
        Mes{i}=zeros(dim*nfens);
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Mes{i} = Mes{i} + (Ns{j}*Ns{j}') * (Jac * mu * weights(j));
        end
    end
    ems  = elematset(struct('mat',{Mes}, 'eqnums',{eqnums}));
end
