% Compute the stiffness matrices of the individual uniaxial gcells.
%
% function ems = stiffness (self, geom, w)
%
% Return an array of them so they may be assembled.
%    Call as
% ems = stiffness(feb, geom, w)
%     geom=geometry field
%     w=displacement field
%
function ems = stiffness (self, geom, w)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    conns = get(gcells, 'conn'); % connectivity
    Kes = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    integration_rule = get(self.feblock, 'integration_rule');
    pc = get(integration_rule, 'param_coords');
    weights  = get(integration_rule, 'weights');
    npts_per_gcell = get(integration_rule, 'npts');
    for j=1:npts_per_gcell
        Ns{j} = bfun(gcells,pc(j,:));
        Nders{j} = bfundpar(gcells,pc(j,:));
    end
    P = self.P; % pre-stressing force
    % Now loop over all gcells in the block
    for i=1:ngcells
        conn =conns(i,:); % connectivity
        eqnums{i} =gather(w,conn,'eqnums');
        x = gather(geom, conn, 'values', 'noreshape'); % coordinates of nodes
        Ke = zeros(dim*nfens); % element stiffness matrix
        % Loop over all integration points
        for j=1:npts_per_gcell
            J = Jacobian_matrix(gcells,Nders{j},x);
            Jac = Jacobian_volume(gcells,conn, Ns{j}, J, x);
            Ndersp = Nders{j}/J;
            B = self.hBlmat(self,Ndersp, [],[]);%Rm(i,:,:));
            Ke = Ke + (B'*P*B) * (Jac * weights(j));
        end
        Kes{i}=Ke;
    end
    ems  = elematset(struct('mat',{Kes}, 'eqnums',{eqnums}));
end
