% Constructor of the finite element block class.
%    This class implements mechanics of small-strain deformation with arbitrarily large
%    deflections and rotations for two-node beam elements.
%
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing options.
% Options: those recognized by feblock
% and 
%      force_function= [];
%      stiffness_function= [];
%      reference_point=[0; 0; 0];
%      direction=[0; 0; 1]; 
function retobj = feblock_defor_tie_line (varargin)
    class_name='feblock_defor_tie_line';
    if (nargin == 0)
        parent = feblock;
        self.force_function= [];
        self.stiffness_function= [];
        self.reference_point=[];
        self.direction=[];
        self.Rayleigh_damping_stiffness=0;
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters=varargin{1};
            parent = feblock(varargin{:});
            gcells=get(parent, 'gcells');
            self.force_function=Parameters.force_function;
            self.stiffness_function=Parameters.stiffness_function;
            self.reference_point=[];
            if (isfield(Parameters,'reference_point'))
                self.reference_point=Parameters.reference_point;
            end
            if (size(self.reference_point,1)==1)
                self.reference_point= ones(count(gcells),1)*self.reference_point;
            end
            self.direction=Parameters.direction;
            if (size(self.direction,1)==1)
                self.direction= ones(count(gcells),1)*self.direction;
            end
            for j=1:size(self.direction,1)
                self.direction(j,:)=self.direction(j,:)/norm(self.direction(j,:));
            end
            self.Rayleigh_damping_stiffness=0;
            if (isfield(Parameters,'Rayleigh_damping_stiffness'))
                self.Rayleigh_damping_stiffness=Parameters.Rayleigh_damping_stiffness;
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
end
