% Compute the conductivity matrices of the individual gcells.
%
% function ems = conductivity(self, geom, temp)
%
% Return an array of the element matrices so they may be assembled.
%    Call as     ems = conductivity(feb, geom, temp)
%    where   geom=geometry field
%            temp=temperature field
function ems = conductivity(self, geom, temp)
    gcells = get(self.feblock,'gcells');
    dim = get(geom,'dim');
    % Material orientation /directions
    Rm = get(self,'Rm');
    eval_Rm = 0;
    if strcmp(class(Rm),'function_handle')
        eval_Rm = true; Rmh =Rm;
    end
    tangents =eye(dim);
    % Material
    mat = get(self.feblock, 'mater');
    kappa = get(get(mat,'property'),'conductivity');
   %     Evaluate the nodal basis function gradients
    bfun_gradients = nodal_bfun_gradients (self, geom);
    %     Preallocate
    Kes= cell(length(bfun_gradients),1);
    eqnums = cell(length(bfun_gradients),1);
    for i=1:length(bfun_gradients)
        eqnums{i} =[];
        Kes{i} = [];
    end
    xs =gather(geom, 1:length(bfun_gradients), 'values', 'noreshape'); % coordinates of all nodes
    eqnumss =gather(temp, 1:length(bfun_gradients), 'eqnums','noreshape');% equation numbers for all nodes
    % Now loop over all finite element nodes
    iem =1;
    for nix=1:length(bfun_gradients)
        if bfun_gradients{nix}.Vpatch>0 % This node has a element patch in this block
            np=length(bfun_gradients{nix}.patchconn);
            c = xs(nix, :); % coordinates of central node
            if (eval_Rm)% do I need to evaluate the local material orientation?
                Rm =Rmh(c,tangents);%RAW: tangent vectors?
            end
            if (isempty(Rm))
                mkappa=kappa;
            else
                mkappa=Rm*kappa*Rm';
            end
            Ke = bfun_gradients{nix}.Nspd*(mkappa*bfun_gradients{nix}.Vpatch)*bfun_gradients{nix}.Nspd' ;
            if (self.stabfact>0)
                lx = xs(bfun_gradients{nix}.patchconn,:); % coordinates of nodes
                Phi =self.hPhi(self,dim,np,lx,c);
                A =Phi'*Phi;
                try
                    A1 =Phi* ((Phi'*Phi)\Phi');
                    Kstab=self.stabfact*Wmean*(eye(size(Ke))-A1);
                    Ke = Ke + Kstab;
                catch
                    warning
                end
            end
            Kes{iem} =  Ke;
            eqnums{iem} =  reshape(eqnumss(bfun_gradients{nix}.patchconn,:)',(np)*size(eqnumss,2),1);
            iem=iem+1;
        end
    end
    ems  = elematset(struct('mat',{Kes(1:iem-1)}, 'eqnums',{eqnums(1:iem-1)}));
end

