% Constructor of the finite element block class.
%
% function retobj = feblock_electrostatics (varargin)
%
% This class implements electrostatics.
%
% See discussion of constructors in <a href="matlab:helpwin 'sofea/classes/Contents'">classes/Contents</a>. 
% Options: those recognized by feblock, plus
% surface_transfer = surface transfer coefficient
%
function retobj = feblock_electrostatics (varargin)
    class_name='feblock_electrostatics';
    if (nargin == 0)
        parent = feblock;
        self.matstates= [];
        retobj = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            options =varargin{1};
            parent = feblock(varargin{:});
            self.matstates= [];
            % Precompute basis functions for quadrature points
            gcells = get(parent,'gcells');
            integration_rule = get(parent, 'integration_rule');
            pc = get(integration_rule, 'param_coords');
            w  = get(integration_rule, 'weights');
            npts_per_gcell = get(integration_rule, 'npts');
            for j=1: npts_per_gcell
                self.N{j} = bfun (gcells(1), pc(j,:));
                self.Nder{j} = bfundpar (gcells(1), pc(j,:));
            end
            retobj = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
