% Compute element load vectors for nonzero essential boundary conditions.
%
% function evs = nz_ebc_loads_electrical_conductivity(self, geom, phi)
%
% Return an array of them so they may be assembled.
%    Call as
% evs = electrical_nz_ebc_loads_conductivity(feb, geom, phi)
%     geom=geometry field
%     phi=Electric potential field
%     temp=Temperature field
%
function evs = nz_ebc_loads_electrical_conductivity(self, geom, phi, temp)
    ems = electrical_conductivity(self, geom, phi, temp);
    gcells = get(self,'gcells');
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(phi,conns(i,:),'eqnums');
        Fe{i} =zeros(length(eqnums{i}),1);
    end
    mat=get(ems,'mat');
    for i=1:size(conns,1)
        conn = conns(i,:); % connectivity
        pT = gather(phi, conn, 'prescribed_values');
        if norm (pT) ~= 0
            Fe{i} =-mat{i}*pT;
        end
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
