% Map a spatial location to parametric coordinates.
%
% function pc = map2parametric(self, x, c, options)
%
% Map a spatial location to parametric coordinates for the geometric cell.
% self = geometric cell, 
% x=array of spatial coordinates of the nodes, size(x) = [nbfuns,dim], 
% c= spatial location (row array), 
% options= Newton's solver control options, such as tolerance and the
% maximum allowable number of iterations
% 
% Note: The dimension of the parametric space must match the space in which
% the spatial coordinates are given.
%
% Returns a row array of parametric coordinates if the solution was
% successful, otherwise NaN are returned.
% 
function pc = map2parametric(self, x, c, options)
    ppc = 0*x(1,:)';
    Tolerance = 0.001;
    maxiter =5;
    pc= ppc;
    for i=1:maxiter
        Nder = bfundpar (self, pc);
        J = Jacobian_matrix (self, Nder, x);
        N = bfun(self,pc);
        R = ((N'*x)-c)';
        pc=ppc -J\R;
        if (norm(pc-ppc,inf) <Tolerance)
            pc=pc';
            return;
        end
        ppc=pc;
    end
    pc= 0*x(1,:)' +nan;
end



