% Reset view for drawing.
%
% function retobj = reset (self, context)
%
% context= structure with optional fields:
% axes = Matlab axes handle: all drawing will go into these axes until they
%      are changed with another reset().
% limits = array of the extents of the drawing
% snap_points = points in 3-D space to which the camera may be targeted. (See the rotate() method.)
function self = reset (self, context)
    if isfield(context,'snap_points')
        self.snap_points= context.snap_points;
    end
    if isfield(context,'axes')
        self.axes= context.axes;
    end
    if (isempty(self.axes))||(self.axes==0)
        self.axes=get(gcf,'CurrentAxes');
        if (isempty(self.axes))
            axes;
            self.axes=gca;
        end
        set(self.axes,'Units','normalized')
        set(self.axes,'position',[0.05  0.1  0.8  0.8]);
        set(self.axes,...
            'Color', 'white', ...
            'FontUnits', 'points', ...
            'FontSize', 14, ...
            'FontAngle', 'Normal', ...
            'FontName', 'Times', ...
            'GridLineStyle', ':', ...
            'Xcolor', [0, 0, 0], ...
            'Ycolor', [0, 0, 0], ...
            'Zcolor', [0, 0, 0]);
        l=get(self.axes,'xlabel');
        set (l,'FontSize',16,...
            'FontAngle', 'Normal', ...
            'FontName', 'Times');
        l=get(self.axes,'ylabel');
        set (l,'FontSize',16,...
            'FontAngle', 'Normal', ...
            'FontName', 'Times');
        l=get(self.axes,'zlabel');
        set (l,'FontSize',16,...
            'FontAngle', 'Normal', ...
            'FontName', 'Times');
        l=get(self.axes,'title');
        set (l,'FontSize',18,...
            'FontAngle', 'Normal', ...
            'FontName', 'Times');
    end
    try
        axes(self.axes); % set the current axes
    catch
        figure(gcf);
        axes;
        self.axes=gca;
    end
    self =clear(self, context);
    if isfield(context,'peek')
        if context.peek
            peekb  = uicontrol('Parent',gcf,'Style','pushbutton',...
                'Units','Normalized','Position',[.9 .0 .1 .05],...
                'String','Peek','Value',0,...
                'TooltipString','Invoke command line in order to peek at data',...
                'Callback',@peek_button_pressed);
        end
    end
    if ~isfield(context,'limits')
        view(3); axis equal vis3d;
        set(self.axes,'XLimMode','auto');
        set(self.axes,'YLimMode','auto');
        set(self.axes,'ZLimMode','auto');
    else
        if length (context.limits) > 4
            axis equal vis3d;
            set(self.axes,'XLim',context.limits(1:2));
            set(self.axes,'YLim',context.limits(3:4));
            set(self.axes,'ZLim',context.limits(5:6));
        else
            view(2); pos=get(self.axes,'position');
            set(self.axes,'XLim',context.limits(1:2));
            set(self.axes,'YLim',context.limits(3:4));
            set(self.axes,'position',pos);
            set(self.axes,'cameraviewanglemode','auto');
        end
        set(self.axes,'DataAspectRatioMode','manual');
        set(self.axes,'DataAspectRatio',[1,1,1]);
    end
    set(gcf,'CurrentAxes',self.axes);
    set(gcf,'Renderer','OpenGL');
    if (ispc) &&  (~strcmp(version('-release'),'13'))
        opengl hardware
    end
    % set(gcf,'DoubleBuffer','on');
    % set(gcf,'Renderer','zbuffer');
    % camlight left;
    % camlight right;
    % camlight headlight;
    % lighting flat;
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
end
    
function peek_button_pressed(h,varargin)
    keyboard;
end
