% Constructor of trapezoidal quadrature rule.
%
% function retobj = trapezoidal_rule (varargin)
%
% The rule is applicable for a tensor product of  intervals -1 <=x<= +1.
% 
% Parameters: 
%   dim=number of  space dimensions
%            3=3-D rule, 2=2-D  rule, 1=1-D rule (the default).
% 
% Example:
%      ir = trapezoidal_rule;
% 
%   Call as:
%      ir = trapezoidal_rule;
%   or
%      ir = trapezoidal_rule(struct('dim',2));
% 
function retobj = trapezoidal_rule (varargin)
    class_name='trapezoidal_rule';
    parent=classbase;
    if (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        end
        if  ~(strcmp(class(varargin{1}),'double'))
            options =varargin{1};
            self.dim = options.dim;
        else
            self.dim = varargin{1};
        end
    else
        self.dim=1;
    end
    switch self.dim
        case 1
            self.param_coords = [-1; 1];
            self.weights = [1, 1];
        case 2
            self.param_coords = [-1,-1; 1,-1; 1, 1; -1, 1];
            self.weights = [1, 1, 1, 1];
        case 3
            self.param_coords = [-1,-1,-1; 1,-1,-1; 1, 1,-1;-1, 1,-1;-1,-1, 1; 1,-1, 1; 1, 1, 1;-1, 1, 1;];
            self.weights = [1, 1, 1, 1, 1, 1, 1, 1];
        otherwise
            error('Unsupported Dimension');
    end
    retobj = class(self,class_name,parent);
end
