% Constructor.
%
% function retobj = property_plastic_perf_j2_iso(varargin)
% 
% This class represents properties of perfect elastic-plastic materials
% whose behavior is described by J2  plasticity
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing parameters.
% Parameters:
% required fields
%     as for property_linel_iso
%   where
%     E       - Young's modulus
%     nu      - Poisson ratio
%     sigma_y - yield stress
% optional fields
%     alpha = coefficient of thermal expansion
%
function retobj = property_plastic_perf_j2_iso(varargin)
    class_name='property_plastic_perf_j2_iso';
    if (nargin == 0)
        parent=property_linel_iso;
        self.sigma_y = 0.0;
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent=property_linel_iso(Parameters);
            self.sigma_y = 0.0;
            if isfield(Parameters,'sigma_y')
                self.sigma_y = Parameters.sigma_y;
                if (self.sigma_y <= 0)
                    error('Non-positive sigma_y!');
                end
            end
            % precompute
             retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
