% Advance the state by specified number of steps
%   Call as:
%      advance(obj, torque, dt, nsteps)
%   where
%      obj       - object
%      torque    - handle to function defining spatial torque
%      dt        - time step
%      nsteps    - number of steps to take
%
%   The first form returns the list of property names as a string,
%   the second form returns the value of the named property.
%
function retobj = advance(self,torque,dt,nsteps)
    Omegan1 = self.Omega;
    Rn1 = self.R;
    t=get(self.rotint,'t');
    I=self.I;
    Pin1=I*Omegan1;
    for i=1:nsteps
        Pin=slv(I, t, dt, Rn1, Pin1, torque, 1000*eps);
        Omegamid=I\(Pin+Pin1)/2;
        Rn=reortho(Rn1*cayley(dt*Omegamid));
        % swap variables for next step
        Rn1=Rn;
        Pin1=Pin;
        t=t+dt;
    end
    self.R = Rn;
    self.Omega = I\Pin;
    self.rotint = set(self.rotint,'t',t);
    retobj=self;
end

function Pin=slv(I, t, dt, Rn1, Pin1, torque, ceps)
    Pin=Pin1;
    i=0; maxi=45;
    tn1 = feval(torque, t, Rn1);
    Tn1 = Rn1'*tn1;
    while (1)
        pPin=Pin;
        Omegamid=I\(Pin+Pin1)/2;
        Rn=(Rn1*cayley(dt*Omegamid));
        tn = feval(torque, t+dt, Rn);
        Pin=Pin1+dt*skewmat(I*Omegamid)*Omegamid+dt/2*(Rn'*tn+Tn1);
        i=i+1;
        if (i > maxi)
            if (max(abs(Pin-pPin)) > 1e-9)
                warning(['akw - Failed to converge: ||residual||=' num2str(norm(Pin))]);
            end
            break;
        end
        if (norm(Pin-pPin) < ceps)
            break;
        end
    end
    return;
end