% Set property in the specified object.
%
%   Call as:
%      a = set(a), or
%      a = set(a,prop_name,val)
%   where
%      a         - object
%      prop_name - property name
%                  To obtain settable property names, use set(a).
%      val       - value to be assigned to the property.
%
%   The first form may be used to obtain a list of settable properties;
%   the second form may be used to set a particular property, and the
%   modified object is returned (do not forget to assign it!).
%
function retval = set(self,varargin)
    if (nargin == 1)
        retval = {{{'I'}, {'tensor of inertia, body frame'}};
            {{'R'}, {'rotation matrix'}};
            {{'Omega'}, {'angular velocity, body frame'}};
            };
        retval = cat(1, get(self.rotint), retval);
        return;
    end
    prop_name=varargin{1};
    val=varargin{2};
    switch prop_name
        case 'I'
            self.I = val;
        case 'R'
            self.R = val;
        case 'omega'
            self.Omega = self.R'*val;
        case 'Omega'
            self.Omega = val;
        case 'zeta'
            self.zeta = val;
        otherwise
            self.rotint = set(self.rotint, prop_name, val);
    end
    retval=self; % return self
end