rho=1.21*1e-9;% mass density
c =343*1000;% millimeters per second
bulk= c^2*rho;
L=500;% length of the box, millimeters
W=200; % cross-sectional area of the box
H=350; % cross-sectional area of the box
graphics= true;% plot the solution as it is computed?
n=8;%
neigvs=5;
OmegaShift=1000;

[fens,gcells] = H8_block(L,W,H,n,n,n); % Mesh

prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
mater=mater_defor_ss_linviscel_triax (struct('property',prop));
feb = feblock_acoustics (struct (...
    'mater',mater,'gcells',gcells,...
    'integration_rule',gauss_rule(3,2)));

geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
P = numbereqns (P);
disp([' Number of equations ' num2str(get(P, 'neqns'))])

S = start (sparse_sysmat, get(P, 'neqns'));
S = assemble (S, acoustic_stiffness(feb, geom, P));
C = start (sparse_sysmat, get(P, 'neqns'));
C = assemble (C, acoustic_mass(feb, geom, P));


[W,Omega]=eigs(get(C,'mat')+OmegaShift*get(S,'mat'),get(S,'mat'),neigvs,'SM');
[Omegas,ix]=sort(diag(Omega)-OmegaShift);
fs=sqrt(Omegas)/(2*pi);

w=0*clone(P,'W');
for i=1:neigvs
    analyt_eigenvalue =sqrt((bulk/rho*((i-1)*pi/L) ^ 2))/2/pi;
    eigenvalue_errors(i) =...
        (fs(i)-analyt_eigenvalue)/analyt_eigenvalue;
    disp(['  Eigenvector ' num2str(i) ...
        ' eigenvalue ' num2str(fs(i)) ' versus '...
        num2str(analyt_eigenvalue)]);
    
    w = scatter_sysvec(w, real(W(:,ix(i))));
    
    
    % Plot
    if graphics
        Camera =1.0e+003 * [-0.856751537046965  -0.753771764316956   1.344954202328690   0.126948462953035   0.086428235683043 0.259454202328690                   0                   0   0.001000000000000   0.016176495044399];
        xyz=get(geom,'values');
        box =reshape([min(xyz);max(xyz)],1,6);
        bg=mesh_bdry(gcells);
        bfeb = feblock(struct ('gcells',bg));
        gv=graphic_viewer;
        gv=reset (gv,[]);
        scale=0.05;
        w = scatter_sysvec(w, W(:,ix(i)));
        wv = get(w,'values');
        dcm=data_colormap(struct ('range',[min(wv),max(wv)], 'colormap',jet));
        colors=map_data(dcm, wv);
        colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
        camset(gv,Camera);
        %         draw(feb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield,'facealpha',0.5,'edgecolor','r', 'shrink', 0.8));
        draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield,'facealpha',0.5));
        for  isovalue=[-0.02,0,0.02]
            draw_isosurface(feb,gv, struct ('x', geom,'u',0*geom,'scalarfield',w,'isovalue',isovalue,'color',map_data(dcm, isovalue)));
        end
        draw_axes (gv,struct('length',20 ));
        pause(0.55);
    end
end