% Hemisphere in water, planar wave directed at an angle to the X. axis
% Structured grid
function savefile=multrb
    frequency=100; angle=0;
    %     pressuremag*R/(rhow*cw^2)./(2*pi*frequency/cw*R)*3/(2*rho/rhow+1);
    R=5.0;% radius of the sphere, millimeters
    nR=3;
    Ro=nR*R; % width of the box, millimeters
    nperR=16;%
    nlayers=nperR;
    rho=1e-9;% mass density
    c = 1.4491e+006;% millimeters per second
    P_amplitude=10^9*1e-6*1e-6;% MPa
    rhos=2.5e-9;% mass density
    %     rhos=rho;% mass density
    dt=1/frequency/20;
    tshift=0;
    tfinal=4*(1/frequency);
    graphics= ~true;% plot the solution as it is computed?
    saveimg= ~true;
    savevariables= ~true;
    tol=R/10000;
    
    bulk= c^2*rho;
    nsteps=round(tfinal/dt)+1;
    omega=2*pi*frequency;
    wavenumber=omega/c;
    front_normal=[cos(pi/180*angle),sin(pi/180*angle),0]';
    wavevector=wavenumber*front_normal;
    uincA=P_amplitude/rho/c/omega;
    postfilename=[mfilename  '-nR' num2str(nR)  '-nperR' num2str(nperR)  '-a' num2str(angle) '-f' num2str(frequency)];
    savefile=[postfilename  '.mat'];
    
    % Load tetrahedral mesh from file
    [fens,gcellsca]=Abaqus_mesh_import('Part1-Study 1.INP',1.0);
    % Classify regions
    gcellsinfo= Classify_regions(fens,gcellsca);
    % Construct the individual rigid bodies and the medium
    clear medium rbs;
    Kounter=0;
    for j=1:length(gcellsca)
        if (gcellsinfo{j}.medium)
            medium=gcellsinfo{j};
            medium.gcells=gcellsca{j};
        else
            Kounter=Kounter+1;
            rbs{Kounter}=gcellsinfo{j};
            feb = feblock (struct ('mater',[], 'gcells',gcellsca{j},...
                'integration_rule', tet_rule(4)));
            geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
            rbs{Kounter}.V=measure(feb,geom,@(x)(1));
            rbs{Kounter}.S=measure(feb,geom,@(x)(x));
            rbs{Kounter}.CG=rbs{Kounter}.S/rbs{Kounter}.V;
            rbs{Kounter}.I=measure(feb,geom,@(x)(skewmat(x-rbs{Kounter}.CG)'*skewmat(x-rbs{Kounter}.CG)));
            rbs{Kounter}.mass=rbs{Kounter}.V*rhos;
            rbs{Kounter}.Inertia=rbs{Kounter}.I*rhos;
            clear feb geom;
        end
    end
    clear gcellsca gcellsinfo;
    % Compress the mesh by deleting the rigid bodies
    connected = find_unconn_fens(fens, medium.gcells);
    [fens, new_numbering] =compact_fens(fens, connected);
    medium.gcells = renumber_gcell_conn(medium.gcells, new_numbering);
    % Construct the boundary of the medium
    bdry_gcells = mesh_bdry(medium.gcells, struct('other_dimension', 1));
    % This is the list of the ABC boundary cells
    ABCl=gcell_select(fens,bdry_gcells,struct ('facing',1,'direction',@(x)(x),'tolerance',0.01));
    % Now we have to find the list of Boundary cells on the boundary of each rigid
    % body
    xyz=get(fens, 'xyz');
    Targetsl=setdiff(1: count(bdry_gcells),ABCl);
    while ~isempty(Targetsl)
        cn= connected_nodes(subset(bdry_gcells,Targetsl));
        rbl=gcell_select(fens,bdry_gcells,struct ('flood', true, 'startfen', cn(1)));
        cn= connected_nodes(subset(bdry_gcells,rbl));
        box=bounding_box(xyz(cn,:));
        for j=1:length(rbs)
            if norm(box-rbs{j}.box)<=10*eps % "The same" box
                rbs{j}.bdry_gcells=subset(bdry_gcells,rbl);
            end
        end
        Targetsl=setdiff(Targetsl,rbl);
    end
    medium.bdry_gcells=subset(bdry_gcells,ABCl);
    clear ABCl bdry_gcells Targetsl rbl;
    % Graphics display?
    if (graphics )
        colors='rbgcmy';
        gv=drawmesh({fens,medium.bdry_gcells},'gcells','facecolor','y','facealpha',0.3)
        for j=1:length(rbs)
            gv=drawmesh({fens, rbs{j}.bdry_gcells},'gv',gv,'gcells','facecolor',colors(j))
        end
        set_graphics_defaults
        labels('x','y','z')
    end
    
    % Now get ready for the computation
    geom = field(struct ('name',['geom'], 'dim', 3, 'fens',fens));
    disp( ['Mesh size: final ' num2str(count(fens))])
    prop=property_viscous_fluid(struct('bulk',bulk,'rho',rho));
    mater=mater_defor_ss_linviscel_triax (struct('property',prop));
    medium.feb = feblock_acoustics (struct (...
        'mater',mater,'gcells',medium.gcells,...
        'integration_rule',tet_rule(4)));
    function n= surface_normal(x,tangent)
        n= cross(tangent(:,1),tangent(:,2));
        n=n/norm(n);
    end
    medium.abcfeb = feblock_acoustics (struct (...
        'mater',mater,'gcells',medium.bdry_gcells,...
        'integration_rule',tri_rule(3),'N',@surface_normal));
    for j=1:length(rbs)
        rbs{j}.sfeb = feblock_acoustics (struct (...
            'mater',mater,'gcells',rbs{j}.bdry_gcells,...
            'integration_rule',tri_rule(3),'N',@surface_normal));
    end
    % Construct the field of pressures
    P = field(struct ('name',['P'], 'dim', 1,'nfens',get(geom,'nfens')));
    P = numbereqns (P);
    tcn=[];
    for j=1:length(rbs)
        tcn= [tcn,connected_nodes(rbs{j}.bdry_gcells)];
    end
    on = setdiff((1:get(geom,'nfens')),tcn);
    permutation=zeros(get(geom,'nfens'),1);
    permutation(tcn)=(1:length(tcn))';
    permutation(on)=(length(tcn)+1:get(P, 'neqns'))';
    P = numbereqns (P,permutation);
    % Construct the system matrices
    S = start (sparse_sysmat, get(P, 'neqns'));
    S = assemble (S, acoustic_stiffness(medium.feb, geom, P));
    mS =get(S,'mat'); clear S
    C = start (sparse_sysmat, get(P, 'neqns'));
    C = assemble (C, acoustic_mass(medium.feb, geom, P));
    mC =get(C,'mat'); clear C
    D = start (sparse_sysmat, get(P, 'neqns'));
    D = assemble (D, surface_acoustic_ABC(medium.abcfeb, geom, P));
    mD =get(D,'mat'); clear D
    
    ForceF = field(struct ('name',['ForceF'], 'dim', 1,'nfens',3));
    ForceF = numbereqns (ForceF);
    
    TorqueF = field(struct ('name',['TorqueF'], 'dim', 1,'nfens',3));
    TorqueF = numbereqns (TorqueF);
    
    mH=0*mC;
    for j=1:length(rbs)
        GF = start (sparse_sysmat, get(ForceF, 'neqns'), get(P, 'neqns'));
        GF = assemble(GF, pressure_to_resultant_force (rbs{j}.sfeb, geom, P, ForceF));
        rbs{j}.mGF =get(GF,'mat'); clear GF
        mH = mH + rbs{j}.mGF'*(rho/rbs{j}.mass)*rbs{j}.mGF;
        GT = start (sparse_sysmat, get(TorqueF, 'neqns'), get(P, 'neqns'));
        GT = assemble(GT, pressure_to_resultant_torque(rbs{j}.sfeb, geom, P, TorqueF, rbs{j}.CG));
        rbs{j}.mGT =get(GT,'mat'); clear GT
        mH = mH + rbs{j}.mGT'*(sparse(rho*inv(rbs{j}.Inertia)))*rbs{j}.mGT;
    end
    
    
    mCtild=mC+mH;
    clear mH
    
    % Solve
    P0 =0*P; % initially all pressure is zero
    clear P
    vP0= gather_sysvec(P0);
    vP1=0*vP0;
    vQ0= 0*vP0;
    vQ1=0*vQ0;
    t=0;
    P1 =P0;
    for j=1:length(rbs)
        rbs{j}.tTa1=[0;0;0];
        rbs{j}.tAa1=[0;0;0];
        rbs{j}.tTa0=rbs{j}.tTa1;
        rbs{j}.tAa0=rbs{j}.tAa1;
    end
    
    fiabc= force_intensity(struct('magn',@abc_pressure_gradient));
    
    pinc =0*P0; % initially all pressure is zero
    pincdd =0*P0; % initially all pressure acceleration is zero
    xyz =get(geom,'values');
    allnl=(1:count(fens));
    function recalculate_incident(t)
        arg=(-xyz*wavevector+omega*(t+tshift));% wave along the x-axis
        vpinc=P_amplitude*sin(arg);
        pinc= scatter(pinc,allnl,vpinc);
        vpincdd=-omega^2*vpinc;
        pincdd= scatter(pincdd,allnl,vpincdd);
        clear vpinc vpincdd
    end
    recalculate_incident(0);
    L0=-mS*gather_sysvec(pincdd)-mCtild*gather_sysvec(pinc);
    F = start (sysvec, get(P1, 'neqns'));
    F = assemble (F, pressure_flux_loads (medium.abcfeb, geom, P1, fiabc));
    La0 =get(F, 'vec');
    tic; disp([' Factorize mA'  ])
    mA = (2/dt)*mS +mD +(dt/2)*mCtild;
    disp(['    nnz mA='  num2str(nnz(mA))])
%     mA = (mA+mA')/2;
%     mA = spdiags(mA,0:-1:-size(mA,1));
    %     [D,dD,prm] = ldl(D,'vector');
    [mA,status,prm] = chol(mA,'lower','vector');
    if ( status ~= 0 ) error,end
    mAt= mA';
    disp(['    Factorized nnz mA='  num2str(nnz(mA))])
    disp(['      Done ' num2str(toc) ])
    
    if ( graphics )
        clear options
        options. box=[-inf,inf,0,inf,-inf,inf];
        options. inflate=(Ro-R)/nlayers/100;
        Halfl=gcell_select(fens, gcells, options);
        hbg = mesh_bdry(subset(gcells,Halfl), struct('other_dimension', 1));
        bfeb = feblock (struct ('mater',[], 'gcells',hbg,...
            'integration_rule', gauss_rule (2,2)));
        gv=graphic_viewer;
    end
    
    rbs{j}.tTv=zeros(3,round(tfinal/dt)+1);
    rbs{j}.tTu=rbs{j}.tTv;rbs{j}.tAv=rbs{j}.tTv;rbs{j}.tAu=rbs{j}.tTv; rbs{j}.incTu=rbs{j}.tTv;
    t=0;
    rbs{j}.incTu(:,1)=-uincA*sin((-rbs{j}.CG*wavevector+omega*(t+tshift)))*front_normal;
    step =0; snap=0;
    while t <=tfinal
        step = step  +1;
        tic; disp([' Time ' num2str(t) ' (' num2str(step) '/' num2str(round(tfinal/dt)+1) ')'])
        if graphics
            Camera =[-135.0272 -173.7116  128.3379    1.9441    4.7929   -1.5659         0         0    1.0000    5.2850];
            wv = get(P0,'values');%pinc+
            dcm=data_colormap(struct ('range',frequency/200000*P_amplitude*[-1,1], 'colormap',bwr));
            colors=map_data(dcm, wv);
            colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
            gv=reset (gv,[]);
            camset(gv,Camera);
            draw(bfeb,gv, struct ('x', geom,'u',0*geom,'colorfield',colorfield,'edgecolor',[1,1,1]/2));
            draw_axes (gv,struct('length',2));
            for  isovalue=linspace(min(get(dcm, 'range')),max(get(dcm, 'range')),20)
                draw_isosurface(bfeb,gv, struct ('x', geom,'u',0*geom,'scalarfield',P0,'isovalue',isovalue,'color',[0,0,0]));
            end
               set_graphics_defaults
            axis off ; %   labels('x','y','z')
            pause(0.0055);
            if saveimg
                saveas(gcf, [mfilename '-' 'p'  '-i' n2s0p(snap) '.png'], 'png');
                snap= snap+1;
            end
        end
        t=t+dt;
        recalculate_incident(t);
        L1=-mS*gather_sysvec(pincdd)-mCtild*gather_sysvec(pinc);
        crhs=(2/dt)*(mS*vQ0)-mD*vQ0-mCtild*(2*vP0+(dt/2)*vQ0)+L0+L1+La0;
        F = start (sysvec, get(P1, 'neqns'));
        F = assemble (F, pressure_flux_loads (medium.abcfeb, geom, P1, fiabc));
        La1 =get(F, 'vec');
        rhs=crhs+La1;
        vQ1(prm) = mAt\(mA\rhs(prm));
        vP1 = vP0 + (dt/2)*(vQ0+vQ1);
        P1 = scatter_sysvec(P1, vP1);
        vp1=gather_sysvec(pinc+P1);
        for j=1:length(rbs)
            Fresultant=rbs{j}.mGF*vp1;
            Tresultant=rbs{j}.mGT*vp1;
            rbs{j}.tTa1=Fresultant/rbs{j}.mass; rbs{j}.tAa1=rbs{j}.Inertia\Tresultant;
        end
        % Reset for the next time step
        vP0 = vP1;
        vQ0 = vQ1;
        P0 = P1;
        L0=L1;
        La0=La1;
        for j=1:length(rbs)
            rbs{j}.tTv(:,step+1)=rbs{j}.tTv(:,step)+dt/2*(rbs{j}.tTa0+rbs{j}.tTa1);
            rbs{j}.tTu(:,step+1)=rbs{j}.tTu(:,step)+dt/2*(rbs{j}.tTv(:,step)+rbs{j}.tTv(:,step+1));
            rbs{j}.tAv(:,step+1)=rbs{j}.tAv(:,step)+dt/2*(rbs{j}.tAa0+rbs{j}.tAa1);
            rbs{j}.tAu(:,step+1)=rbs{j}.tAu(:,step)+dt/2*(rbs{j}.tAv(:,step)+rbs{j}.tAv(:,step+1));
            rbs{j}.incTu(:,step+1)=-uincA*cos((-rbs{j}.CG*wavevector+omega*(t+tshift)))*front_normal;
            rbs{j}.tTa0=rbs{j}.tTa1;
            rbs{j}.tAa0=rbs{j}.tAa1;
        end
    end% time integration loop
    rbs{j}.tTu=rbs{j}.tTu (:,1:step);
    rbs{j}.tAu=rbs{j}.tAu (:,1:step);
    rbs{j}.incTu=rbs{j}.incTu (:,1:step);
    clear mS mC mD mA mAt
    if ( savevariables )
        save (savefile);
    end
    
    uamplHW=P_amplitude/(rho*c^2)./(omega/c)*3/(2*rhos/rho+1);
    for j=1:length(rbs)
        t=linspace(0,size(rbs{j}.tTu,2)*dt,size(rbs{j}.tTu,2));
        uxa=amplitude(t,remove_bias(t,rbs{j}.tTu(1,:)));
        disp( ['Normalized Amplitude of the x-Displ.  of the CG=' num2str(uxa/uamplHW)])
        figure;
        hold on
        plot(t,remove_bias(t,rbs{j}.tTu(1,:))/uamplHW,'kx-','linewidth',2)
        %     plot(t,remove_bias(t,tTu(2,:))/uamplHW,'gx--','linewidth',2)
        %     plot(t,remove_bias(t,tTu(3,:))/uamplHW,'bx:','linewidth',2)
        grid on
        xlabel(' Time'); ylabel('Normalized x-Displ. of the CG'); set_graphics_defaults
        figure;
        hold on
        plot(t,rbs{j}.tTu(1,:),'kx-','linewidth',2)
        %     plot(t,tTu(2,:),'gx--','linewidth',2)
        %     plot(t,tTu(3,:),'bx:','linewidth',2)
        grid on
        xlabel(' Time'); ylabel('x-Displacement of the CG'); set_graphics_defaults
        figure
        hold on
        %     plot(t,tAu(1,:),'rx-','linewidth',2)
        plot(t,rbs{j}.tAu(2,:),'kx--','linewidth',2)
        %     plot(t,tAu(3,:),'bs:','linewidth',2)
        grid on
        xlabel(' Time'); ylabel('y-Rotation about the CG'); set_graphics_defaults
        figure;
        hold on
        %     plot(t,remove_bias(t,tAu(1,:))*R/uxa,'rx-')
        plot(t,remove_bias(t,rbs{j}.tAu(2,:))*R/uxa,'kx--','linewidth',2)
        %     plot(t,remove_bias(t,tAu(3,:))*R/uamplHW,'bx:')
        grid on
        xlabel(' Time'); ylabel('Normalized z-Displ. Amplitude'); set_graphics_defaults
        rya=amplitude(t,remove_bias(t,rbs{j}.tAu(2,:)));
        disp( ['Amplitude of the y-Rot. about the CG=' num2str(rya)])
        disp( ['Normalized z-Displ. Amplitude=' num2str(rya*R/uxa)])
    end
    
    %     visualize_motion
    
    function val=target_pressure_gradient(x,J)
        n=cross(J(:,1),J(:,2)); n=n/norm(n);
        val=-rho*dot(cross(tAa1,x-CG)+tTa1',n);
    end
    function val=abc_pressure_gradient(x,J)
        n=cross(J(:,1),J(:,2)); n=n/norm(n);
        arg=(-x*wavevector+omega*(t+tshift));
        val=P_amplitude*cos(arg)*(-dot(n,wavevector));
    end
    
    function p= find_peaks(t,y)
        p=[];
        dy=diff(y);
        for j3=1:length(dy)-1
            if (dy(j3)*dy(j3+1)<0)
                p(end+1)=y(j3+1);
            end
        end
    end
    function A= amplitude(t,y)
        p= find_peaks(t,y) ;
        A=mean(abs(p)) ;
    end
    
    function visualize_motion
        Views{1}.Tag='iso';
        Views{1}.Camera=[75.8148   85.1701   74.2256    2.8175    2.6616    5.3870         0         0    1.0000    5.9140];
        Views{2}.Tag='Y';
        Views{2}.Camera=[0.6605  132.4248    4.5452   -0.4731    2.5259    4.5452         0         0    1.0000    5.9140];
        for jV=1:length(Views)
            View= Views{jV};
            scale=25000*frequency/500;
            saveimg=1;
            moviefilename=[postfilename '-' 'u' '-V' View.Tag];
            
            
            xyz=get(fens, 'xyz');
            bfeb = feblock(struct ('gcells',target_gcells));
            
            gv=graphic_viewer;
            
            t=linspace(0,size(tTu,2)*dt,size(tTu,2));
            Us=tTu;
            Rs=tAu;
            
            % Us=0*tTu;
            % Us(1,:)=remove_bias(t,tTu(1,:));
            % Us(1,:)=Us(1,:)-Us(1,1);
            % Us(1,:)=Us(1,:)-(incTu(1,:));
            % Us(2,:)=remove_bias(t,tTu(2,:));
            % Us(2,:)=Us(2,:)-Us(2,1);
            % Us(2,:)=Us(2,:)-(incTu(2,:));
            % Us(3,:)=remove_bias(t,tTu(3,:));
            % Us(3,:)=Us(3,:)-Us(3,1);
            % Us(3,:)=Us(3,:)-(incTu(3,:));
            % Rs=0*tAu;
            % Rs(1,:)=remove_bias(t,tAu(1,:));
            % Rs(1,:)=Rs(1,:)-Rs(1,1);
            % Rs(2,:)=remove_bias(t,tAu(2,:));
            % Rs(2,:)=Rs(2,:)-Rs(2,1);
            % Rs(3,:)=remove_bias(t,tAu(3,:));
            % Rs(3,:)=Rs(3,:)-Rs(3,1);
            snap=0;
            for j=1:size(tTu,2)
                U=xyz*0;
                for k=1:size(xyz,1)
                    U(k,:)=(scale*Us(:,j)+cross(scale*Rs(:,j),(xyz(k,:)-CG)'));
                end
                gv=reset (gv,[]);
                set_graphics_defaults;
                camset(gv,View.Camera);
                u= field(struct ('name',['u'], 'dim', 3, 'data',U));
                draw(bfeb,gv, struct ('x', geom,'u',0*u,'facecolor','none','edgecolor',[1,1,1]*0.6,'linewidth',1));
                draw(bfeb,gv, struct ('x', geom,'u',u,'facecolor','w','edgecolor','k','linewidth',2));
                %     draw_axes (gv,struct('length',4));
                draw_planes(gv, struct( 'origin',CG, 'length',2*R,'fontname', 'Times','fontsize',16));
                grid off, axis off
                %             headlight(gv);
                pause(0.0055);
                if saveimg
                    saveas(gcf, [moviefilename '-i' n2s0p(snap) '.png'], 'png');
                    system(['convert -resize 28% '  [moviefilename '-i' n2s0p(snap) '.png'] ' ' [moviefilename '-i' n2s0p(snap) '.eps']]);
                    %             saveas(gcf, [op{1} '-' 'Tiso'  '-t' n2s0p(pt) '.png'], 'png');
                    %                 system ([' convert -resize 28% ' [postfile '-' 'Tiso'  '-i' n2s0p(img) 'nat.png'] ' ' [postfile '-' 'Tiso'  '-i' n2s0p(img) '-smallnat.png']])
                    snap= snap+1;
                end
            end
            system(['convert '   moviefilename '*-i*' '.png' ' ' moviefilename '.gif']);
        end
    end
    
    function gcellsinfo= Classify_regions(fens,gcellsca)
        xyz=get(fens, 'xyz');
        clear gcellsinfo;
        for j=1:length(gcellsca)
            cn= connected_nodes(gcellsca{j});
            gcellsinfo{j}.label =  unique(get(gcellsca{j},'label'));
            gcellsinfo{j}.box = bounding_box (xyz(cn,:));
            gcellsinfo{j}.medium = false;
        end
        
        for j=1:length(gcellsca)
            gcellsinfo{j}.medium=true;
            for k=1:length(gcellsca)
                gcellsinfo{j}.medium= gcellsinfo{j}.medium ...
                    && in_box (gcellsinfo{j}.box,gcellsinfo{k}.box([1,3,5]))...
                    && in_box (gcellsinfo{j}.box,gcellsinfo{k}.box([2,4,6]));
            end
        end
    end
    
end