%%  Straight cantilever, transverse vertical force at the tip
%
% Cantilevered beam with transverse vertical force of the tip. 

function NS_simplest
     %%
    %
    % Parameters:
    E=1e7;% lb.in^-2
    h= 1; b =1;% in
    Length=100;% in
    utol=1e-6;
    Fmag=0.8*E*h*b/Length;
    
    % Mesh
    X= [0,0,0;0, Length,-Length/100];
    Parameters.other_dimension=1;
    [fens,gcells] = Springs(X,[1,2],Parameters);
    basen=fenode_select(fens, struct('box', update_box([],X(1,:))));
    tipn=fenode_select(fens, struct('box', update_box([],X(2,:))));
    
    
    % Finite element block
    function k= Stiffness(L0,L1)
        k=E*h*b/L0;
    end
    function k= Force(L0,L1)
        k=E*h*b*(L1-L0)/L0;
    end
    feb = feblock_defor_spring(struct ('gcells',gcells,...
        'Stiffness_function',@Stiffness, 'Force_function',@Force));
    % Initialize configuration variables
    geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
    u0=0*geom0;
    % Define the displacement increment field
    dchi  = field(struct ('name',['dchi'], 'dim', 3, 'data', zeros(get(geom0,'nfens'),3)));
    
    % Apply EBC's
    ebc_fenids=[basen];
    ebc_prescribed=1;
    ebc_comp=[1,2,3];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    ebc_fenids=[tipn];
    ebc_prescribed=1;
    ebc_comp=[1,2];
    ebc_val=0;
    dchi = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
    dchi = apply_ebc (dchi);
    % Number equations
    dchi = numbereqns (dchi);
    neqns=get(dchi, 'neqns');
    
    gv=graphic_viewer;
    
    
    load_parameters=0.05:0.05:1;
    tip_displacement=[];
    for load_parameter=load_parameters
        
        % Initialization
        dchi = 0*dchi; % guess
        dchi = apply_ebc(dchi);
        u1 = u0;
        nl = nodal_load(struct ('id',tipn,'dir',[3],'magn',[-load_parameter*Fmag]));
        
        iter=1;
        while 1
            feb1=feb;  % Make a copy of the state
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,u1,dchi);       % Internal forces
            F = assemble (F, evs);
            K = start (sparse_sysmat, neqns);
            K = assemble (K, cat(2,stiffness(feb1,geom0,u1,dchi),stiffness_geo(feb1,geom0,u1,dchi)));
            dchi = scatter_sysvec(dchi, K\F); % Displacement increment
            % Line search
            R0 = dot(get(F,'vec'),gather_sysvec(dchi));
            F = start (sysvec, neqns);
            F = assemble (F, loads(nl, dchi));
            [feb1,evs] = restoring_force(feb1,geom0,...
                u1+slice(dchi,(1:3),'transl'),dchi);       % Internal forces
            F = assemble (F, evs);
            R1 = dot(get(F,'vec'),gather_sysvec(dchi));
            a = R0/R1;
            if ( a<0 )
                eta = a/2 +sqrt((a/2)^2 -a);
            else
                eta =a/2;
            end
            eta=min( [eta, 1.0] );
            u1 = u1 + eta*slice(dchi,(1:3),'transl');   % increment displacement
            disp(['   It. ' num2str(iter) ': ||du||=' num2str(norm(dchi))]);
            if (max(abs(gather(dchi,[],'values'))) < utol) break; end;                    % convergence check
            iter=iter+1;
        end
        [feb,ignore] = restoring_force(feb,geom0,u1,dchi);        % final update
        disp(['    Converged for load_parameter=' num2str(load_parameter)]); % pause
        u0 = u1; % update the displacement
        utip=gather(u1,get(nl, 'id'),'values');
        disp(['Tip deflection =' num2str(utip(1:3)','%27.15e, %27.15e, %27.15e\n')])
        tip_displacement=[tip_displacement;utip(1:3)'];
        
        %%
        % Graphic display of the mesh and the deformed beam
        gv=reset (gv,[]);
        scale=1;
        draw(feb, gv, struct ('x', geom0, 'u', [], 'facecolor','y'));
        draw(feb, gv, struct ('x', geom0, 'u', u1,  'facecolor',[1,1,1]/2));
        %     draw_local_coordinates(feb, gv, struct ('x', geom, 'u', scale*u, 'rot', 0*rot, 'length',25));
        
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        hold on
        
        %     draw(fens,gv, struct ('x', geom, 'u', 0*u, 'color','blue','offset',0.35));
        draw_axes(gv, struct([]));
        camset (gv, [854.0279  -90.7814  183.7267   15.8205   49.4864  -28.1679         0         0    1.0000    5.0831]);
            pause(.1)
    end
    
    %     % figure
    %     % plot(izzelna_ux(:,1),izzelna_ux(:,2),'r-','linewidth',3)
    %     % hold on
    %     % plot(izzelna_uy(:,1),izzelna_uy(:,2),'r-','linewidth',3)
    %     % hold on
    %     % plot(izzelna_uz(:,1),izzelna_uz(:,2),'r-','linewidth',3)
    %     % hold on
    %     % plot(tip_displacement(:,1),load_parameters*Fmag,'x','linewidth',3)
    %     % plot(tip_displacement(:,2),load_parameters*Fmag,'x','linewidth',3)
    %     % plot(tip_displacement(:,3),load_parameters*Fmag,'x','linewidth',3)
    %     % title('Curved cantilever, L-to-R uy,ux,uz','fontname','Times','fontsize',16)
    %     % labels(' Tip displacement [in]','Load [lb]')
    %     % grid on
    %     % set_graphics_defaults
    %     % 
end