% Extrude a mesh of quadrilaterals into a mesh of hexahedra (H8).
%
% function [fens,gcells] = Q4_extrude(fens,gcells,nLayers,extrusionh)
%
% Inputs:
% fens,gcells = nodes and cells of the quadrilateral mesh,
% nLayers= number of layers,
% extrusionh= handle to function that computes the extrusion vector
%   nxyz=extrusionh(xyz,k)
% 
% Output:
% fens= node set which incorporates the nodes on input plus the nodes of
% the extruded elements. Note that no merging (fusing) of nodes is performed.
% h8gcells = H8 extruded geometric cells
function [h8fens,h8gcells] = Q4_extrude(fens,gcells,nLayers,extrusionh)
    id = get(fens,'id');
    cn=connected_nodes(gcells);
    id(cn)=(1:length(cn))';
    q4gcells =update_connectivity(gcells,id);
    xyz=get(fens,'xyz');
    q4fens =fenodeset(struct('id',(1:length(cn))','xyz',xyz(cn,:)));
    
    [h8fens,h8gcells]= do_extrude(q4fens,q4gcells,nLayers,extrusionh);
%     [fens,efens,fens2] = fuse_nodes(efens, fens, 0);
%     id = get(efens,'id');
%     h8gcells =update_connectivity(egcells,id);
end

function [efens,egcells]= do_extrude(fens,gcells,nLayers,extrusionh)
    cn=connected_nodes(gcells);
    xyz1=get(fens,'xyz');
    nn1 =count(fens);
    nnt=nn1*nLayers;
    ngc=count(gcells)*nLayers;
    hconn=zeros(ngc,8);
    xyz1=get(fens,'xyz');
    xyz =zeros(size(xyz1,1)*(nLayers+1),3);
    for j=1:length(cn)
        xyz(cn(j),:) =extrusionh(xyz1(cn(j),:),0);
    end
    for k=1:nLayers
        for j=1:length(cn)
            f=cn(j)+k*nn1;
            xyz(f,:) =extrusionh(xyz1(cn(j),:),k);
        end
    end
    
    conn = get(gcells,'conn');
    gc=1;
    for k=1:nLayers
        for i=1:count(gcells)
            hconn(gc,:)=[conn(i,:)+(k-1)*nn1, conn(i,:)+k*nn1];
            gc=gc+1;
        end
    end
    egcells =gcellset_H8(struct('id',1:ngc,'conn',hconn));
    efens =fenodeset(struct('id',1:size(xyz,1),'xyz',xyz));
end